#!/usr/bin/env bash

DIR=$(dirname "$(readlink -f "$0")")

source $DIR/pegasus_common.sh

function compute() {
  algorithm=$1
  graph=$2
  
  input=pegasus/graphs/$graph
  partitioned=pegasus/graphs-partitioned/$graph
  
  case "$algorithm" in
    "pagerank")
      echo -n "Enter parameters: [# of blocks] [threshold for dense regions] [damping factor]: "
      read blocks threshold alpha
      $DIR/pagerank_hadoop.sh $input $partitioned pegasus/pagerank/$graph $threshold $blocks -1 $alpha
      echo "[PageRank] the PageRank result for $graph graph is stored in HDFS."
      ;;
    "rwr")
      echo -n "Enter parameters: [# of blocks] [threshold for dense regions] [damping factor] [source vertex]: "
      read blocks threshold alpha source_vertex
      $DIR/pagerank_hadoop.sh $input $partitioned pegasus/rwr/$graph $threshold $blocks -1 $alpha $source_vertex
      echo "[RWR] the RWR result for $graph graph is stored in HDFS."
      ;;
    "sssp")
      echo -n "Enter parameters: [# of blocks] [threshold for dense regions] [source vertex]: "
      read blocks threshold source_vertex
      $DIR/sssp_hadoop.sh $input $partitioned pegasus/sssp/$graph $threshold $blocks $source_vertex -1
      echo "[SSSP] the shortest path computation result for $graph graph is stored in HDFS."
      ;;
    "diameter")
      echo -n "Enter parameters: [# of blocks] [threshold for dense regions]"
      read blocks threshold
      $DIR/diameter_hadoop.sh $input $partitioned pegasus/diameter/$graph $threshold $blocks -1
      echo "[Diameter] the shortest path computation result for $graph graph is stored in HDFS."
      ;;
    "triangle")
      echo -n "Enter parameters: [# of colors] [counting or enumeration]"
      read num_colors mode
      if [[ $mode == "counting" ]]; then
        $DIR/pte_hadoop.sh $num_colors $input
        echo "[PTE] the triangle counting for $graph graph is completed."
      else
        $DIR/pte_hadoop.sh $num_colors $input pegasus/triangle/$graph
        echo "[PTE] the triangle enumeration result for $graph graph is stored in HDFS."
      fi
      ;;
    "subgraph")
      echo -n "Enter parameters: [# of colors] [pattern] [counting or enumeration] [# of workers]"
      read num_colors pattern_str mode workers
      if [[ $mode == "counting" ]]; then
        $DIR/pse_hadoop.sh $num_colors $input $pattern_str $workers
        echo "[PSE] the subgraph counting for $graph graph is completed."
      else
        $DIR/pse_hadoop.sh $num_colors $input $pattern_str $workers pegasus/subgraph/$graph
        echo "[PSE] the subgraph enumeration result for $graph graph is stored in HDFS."
      fi
      ;;
    "cc")
      echo -n "Enter parameters: [# of partitions] [local threshold]: "
      read workers threshold
      $DIR/pacc_hadoop.sh $input pegasus/cc/$graph $workers $threshold
      echo "[PACC] the connected component computation result for $graph graph is stored in HDFS."
      ;;
  esac
}

function generate() {
  model=$1
  
  echo -n "Enter parameters: [graph name] [parallelism]: "
  read graph parallelism
  output=pegasus/graphs/$graph
  
  case "$model" in 
    "rmat")
      echo -n "Enter model parameters: [level of recur.] [# of edges] [a] [b] [c] [d] [noise]: "
      read level edges a b c d noise
      
      $DIR/teg_rmat_hadoop.sh $level $edges $a $b $c $d $noise $output
      
      hdfs dfs -rm -r $output.input
      
      echo "[TeG-RMAT] The generated R-MAT graph $graph is stored in HDFS."
      ;;
    "random")
      echo -n "Enter model parameters: [# of nodes] [# of edges]: "
      read num_nodes num_edges
      $DIR/teg_rand_hadoop.sh $num_nodes $num_edges $parallelism $output
      
      echo "[TeG-Random] The generated random graph $graph is stored in HDFS."
      ;;
    "kronecker")
      echo -n "Enter model parameters: [seed graph] [level of recur.]: "
      read seed level
      seed_input=pegasus/graphs/$seed
      $DIR/teg_krnk_hadoop.sh $seed_input $level $parallelism $output
      
      echo "[TeG-Kronecker] The generated Kronecker graph $graph is stored in HDFS."
  esac
}


function plot() {
  echo "";
}

print_intro
main_loop
