#!/usr/bin/env bash

# get current path
DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"

# get binary path of PegasusG on Hadoop
BINARY_NAME=pegasus-mapreduce-assembly-0.1.0-SNAPSHOT.jar
BINARY_PATH=$DIR/../mapreduce/target/$BINARY_NAME

# path on local
GRAPH_DIR=$DIR/graphs
TOOLS_DIR=$DIR/../tools

# paths on HDFS
GRAPH_ON_HDFS=pegasusg-graph
PARTITIONED_GRAPH_ON_HDFS=pegasusg-partitioned
DIAMETER_ON_HDFS=pegasusg-diameter

# check whether binary of PegasusG on Hadoop exists or not
if [ ! -f "$DIR/../mapreduce/target/$BINARY_NAME" ]; then
  echo "PegasusG binary on Hadoop not found! Build the binary..."
  $TOOLS_DIR/sbt clean assembly
fi

hdfs dfs -rm -r $GRAPH_ON_HDFS $DIAMETER_ON_HDFS
hdfs dfs -put $GRAPH_DIR/fb.tsv $GRAPH_ON_HDFS

# prepartitioning
hadoop jar $BINARY_PATH pegasus.mapreduce.common.prepartitioning.UnweightedGraphPrepartitioning \
 -D mapreduce.job.reduces=3 -D pmv.io.denseThreshold=30 $GRAPH_ON_HDFS $PARTITIONED_GRAPH_ON_HDFS directed edgelist

# run diameter
hadoop jar $BINARY_PATH pegasus.mapreduce.diameter.DiameterComputation \
 -D pmv.io.denseThreshold=30 $PARTITIONED_GRAPH_ON_HDFS $DIAMETER_ON_HDFS 3 30

# clear unused files
hdfs dfs -rm -r $GRAPH_ON_HDFS $PARTITIONED_GRAPH_ON_HDFS
