/*
 * PegasusN: Peta-Scale Graph Mining System (Pegasus v3.0)
 * Authors: Chiwan Park, Ha-Myung Park, U Kang
 *
 * Copyright (c) 2018, Ha-Myung Park, Chiwan Park, and U Kang
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Seoul National University nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * -------------------------------------------------------------------------
 * File: Constants.java
 * - Constants for PegasusG MapReduce Implementation
 * Version: 3.0
 */
package pegasus.mapreduce.common;

public class Constants {
  public static final String EdgeListDelimiter = "pmv.io.delimiter.edgeListPair";
  public static final String AdjacencyListVertexDelimiter = "pmv.io.delimiter.adjacencyList.vertex";
  public static final String AdjacencyListEdgeDelimiter = "pmv.io.delimiter.adjacencyList.edge";

  public static final String StatPath = "_STATS";

  public static final String DenseThreshold = "pmv.io.denseThreshold";
  public static final int DefaultDenseThreshold = 100;

  public static final String SizeOfVector = "pmv.vector.size";

  public static final int InitArrayWritableSize = 1024;

  public static final String Parallelism = "pmv.parallelism";
  public static final String ExecutedIterations = "executedIterations";
  public static final String MaxIterations = "pmv.maxIterations";

  public static final int SizeOfMemoryBarrier = 256 * 1024 * 1024;

  public static final int MaxNumWorkers = 1024;
  public static final int CoordinationTickTime = 3000;
  public static final String BarrierRoot = "/pmv-barrier";
  public static final String CounterRoot = "/pmv-counter";
  public static final String CounterGroup = "pmv.counter";

  public static final String DFSTemporaryPath = "/tmp";

  public static final String SparseVectorKey = "sv";
  public static final String DenseVectorKey = "dv";
  public static final String IntermediateVectorKey = "pv";

  public static final double DoubleValueZeroThreshold = 1e-7;

  public static final String PageRankDampingFactor = "pmv.pagerank.dampingFactor";
  public static final double DefaultPageRankDampingFactor = 0.85;
  public static final String PersonalizedPageRank = "pmv.pagerank.personalized";
  public static final String PageRankSourceVertex = "pmv.pagerank.sourceVertex";
  public static final String PageRankSumCounter = "pmv.pagerank.rankSum";
  public static final String PageRankSumBarrier = "pagerank-sum";
  public static final String PageRankThreshold = "pmv.pagerank.threshold";
  public static final double DefaultPageRankThreshold = 1e-8;

  public static final String SSSPSourceVertex = "pmv.sssp.sourceVertex";

  public static final double PHI = 0.77351;
  public static final String DiameterNumIterations = "pmv.diameter.numIterations";
  public static final String DiameterNeighborhood = "diameter.neighborhood";
  public static final String DiameterNeighborhoodFile = "pmv-diameter-neighborhood";

  public static final int DefaultNumBitstrings = 32;

  public static final String DistributionMinValue = "pmv.distribution.minValue";
  public static final String DistributionMaxValue = "pmv.distribution.maxValue";
  public static final String DistributionNumBins = "pmv.distribution.numBins";
  public static final int DistributionDefaultNumBins = 1000;
}
