/*
 * PegasusN: Peta-Scale Graph Mining System (Pegasus v3.0)
 * Authors: Chiwan Park, Ha-Myung Park, U Kang
 *
 * Copyright (c) 2018, Ha-Myung Park, Chiwan Park, and U Kang
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Seoul National University nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * -------------------------------------------------------------------------
 * File: VectorCache.java
 * - Interface for cache of vector elements
 * Version: 3.0
 */
package pegasus.mapreduce.common.cache;

import org.apache.hadoop.io.Writable;

public interface VectorCache<V extends Writable> {
  /**
   * Returns the size of vector.
   *
   * @return The size of vector
   */
  int size();

  /**
   * Retrieves the vector element in the given index.
   *
   * @param i The index of element to be retrieved
   * @return The vector element
   */
  V get(final int i);

  /**
   * Returns whether the i-th index is in the vector or not.
   *
   * @param i The index to be checked
   * @return true if the i-th element is in the vector, otherwise false
   */
  boolean contains(final int i);

  /**
   * Puts a vector element.
   *
   * @param i     The index of element
   * @param value The value to be put
   */
  void put(final int i, final V value);

  /**
   * Adds an element to a specific index.
   *
   * @param i     The index of element
   * @param value The value to be added
   */
  void addTo(final int i, final V value);

  /**
   * Clears the vector cache.
   */
  void clear();

  /**
   * Returns an iterator object that iterates all elements.
   *
   * @param onlyNonZero The flag to iterate only non-zero elements
   * @param filterSet   The set to filter elements to be iterated
   * @return An iterator object
   */
  VectorIterator<V> iterator(boolean onlyNonZero, final BitSet filterSet);
}
