/*
 * PegasusN: Peta-Scale Graph Mining System (Pegasus v3.0)
 * Authors: Chiwan Park, Ha-Myung Park, U Kang
 *
 * Copyright (c) 2018, Ha-Myung Park, Chiwan Park, and U Kang
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Seoul National University nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * -------------------------------------------------------------------------
 * File: ZookeeperBooleanCounter.java
 * - A counter implementation that contains boolean value
 * Version: 3.0
 */
package pegasus.mapreduce.common.coordination;

import org.apache.zookeeper.ZooKeeper;

import java.util.function.BinaryOperator;

public class ZookeeperBooleanCounter extends ZookeeperCounter<Boolean> {
  public ZookeeperBooleanCounter(ZooKeeper zk, String name, int workerId) {
    super(zk, name, workerId);
  }

  @Override
  protected Boolean negativeInfinite() {
    return false;
  }

  @Override
  protected Boolean positiveInfinite() {
    return true;
  }

  @Override
  protected Boolean zero() {
    return false;
  }

  @Override
  protected BinaryOperator<Boolean> sumOp() {
    return (a, b) -> a | b;
  }

  @Override
  protected Boolean valueFromString(String string) {
    return "true".equalsIgnoreCase(string);
  }

  @Override
  protected String valueToString(Boolean value) {
    return value.toString();
  }
}
