/*
 * PegasusN: Peta-Scale Graph Mining System (Pegasus v3.0)
 * Authors: Chiwan Park, Ha-Myung Park, U Kang
 *
 * Copyright (c) 2018, Ha-Myung Park, Chiwan Park, and U Kang
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Seoul National University nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * -------------------------------------------------------------------------
 * File: LocalPaths.java
 * - Helper class for managing local paths.
 * Version: 3.0
 */

package pegasus.mapreduce.common.io;

import java.io.File;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.Arrays;
import java.util.Random;

import org.apache.log4j.Logger;

public class LocalPaths {

	private final static Logger LOGGER = Logger.getLogger(LocalPaths.class.getName());
	
	private static final Random rand = new Random();
	
	private static String[] basePaths;
	
	public static void setBasePaths(String[] basePaths){
		
		LOGGER.info("Basepaths: " + Arrays.toString(basePaths));
		
		for(String tmpDir : basePaths){
			new File(tmpDir).mkdirs();
		}
		
		LocalPaths.basePaths = basePaths;
	}
	
	private static String getNextBasePath(){
		return basePaths[rand.nextInt(basePaths.length)];
	}
	
	public static String getTempPath(String prefix){
		
		String res = null;
		do{
			int t = rand.nextInt(Integer.MAX_VALUE);
			res = getNextBasePath() + "/" + prefix + "-" + t;
		}while(Files.exists(Paths.get(res)));
		
		return res;
		
	}
	
}
