/*
 * PegasusN: Peta-Scale Graph Mining System (Pegasus v3.0)
 * Authors: Chiwan Park, Ha-Myung Park, U Kang
 *
 * Copyright (c) 2018, Ha-Myung Park, Chiwan Park, and U Kang
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Seoul National University nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * -------------------------------------------------------------------------
 * File: IntDoublePairWritable.java
 * - Represents an (int, double) pair value
 * Version: 3.0
 */
package pegasus.mapreduce.common.types;

import org.apache.hadoop.io.WritableComparable;
import org.apache.hadoop.io.WritableComparator;
import pegasus.mapreduce.common.utils.TypeUtils;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;

/**
 * Represents an (int, double) pair value.
 */
public class IntDoublePairWritable implements WritableComparable<IntDoublePairWritable> {
  private int intValue;
  private double doubleValue;

  public IntDoublePairWritable() {
  }

  public IntDoublePairWritable(final int intValue, final double doubleValue) {
    set(intValue, doubleValue);
  }

  public int getInt() {
    return intValue;
  }

  public double getDouble() {
    return doubleValue;
  }

  public void set(final int intValue, final double doubleValue) {
    this.intValue = intValue;
    this.doubleValue = doubleValue;
  }

  @Override
  public void write(final DataOutput out) throws IOException {
    out.writeInt(intValue);
    out.writeDouble(doubleValue);
  }

  @Override
  public void readFields(final DataInput in) throws IOException {
    intValue = in.readInt();
    doubleValue = in.readDouble();
  }

  @Override
  public int compareTo(final IntDoublePairWritable other) {
    return intValue < other.intValue ? -1 : intValue > other.intValue ? 1 : 0;
  }

  static {
    WritableComparator.define(
        IntDoublePairWritable.class, new TypeUtils.IntComparator(IntDoublePairWritable.class));
  }
}
