/*
 * PegasusN: Peta-Scale Graph Mining System (Pegasus v3.0)
 * Authors: Chiwan Park, Ha-Myung Park, U Kang
 *
 * Copyright (c) 2018, Ha-Myung Park, Chiwan Park, and U Kang
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Seoul National University nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * -------------------------------------------------------------------------
 * File: IDUtils.java
 * - Provides ID conversion methods between local vertex ID and global vertex ID.
 * Version: 3.0
 */
package pegasus.mapreduce.common.utils;

/**
 * Provides some utility methods about vertex IDs.
 */
public class IDUtils {
  /**
   * Converts a global vertex ID to local vertex ID.
   *
   * @param numBlocks The number of total vector blocks
   * @param globalId  A global ID of vertex
   * @return A local ID of the given vertex
   */
  public static int globalToLocal(int numBlocks, long globalId) {
    return (int) (globalId / numBlocks);
  }

  /**
   * Converts a local vertex ID to global vertex ID.
   *
   * @param numBlocks The number of total vector blocks
   * @param blockId   An ID of vector block who contains the vertex
   * @param localId   A local ID of vertex
   * @return A global ID of the given vertex
   */
  public static long localToGlobal(int numBlocks, int blockId, int localId) {
    return ((long) localId) * numBlocks + blockId;
  }
}
