/*
 * PegasusN: Peta-Scale Graph Mining System (Pegasus v3.0)
 * Authors: Chiwan Park, Ha-Myung Park, U Kang
 *
 * Copyright (c) 2018, Ha-Myung Park, Chiwan Park, and U Kang
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Seoul National University nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * -------------------------------------------------------------------------
 * File: MemoryUtils.java
 * - Provides memory utility methods to inspect current memory status.
 * Version: 3.0
 */
package pegasus.mapreduce.common.utils;

/**
 * Provides some memory utility methods to inspect current memory status.
 */
public class MemoryUtils {
  private static final Runtime JavaRuntime = Runtime.getRuntime();

  /**
   * Call garbage collector 5 times explicitly.
   */
  public static void callGCMultipleTimes() {
    callGCMultipleTimes(5);
  }

  /**
   * Call garbage collector explicitly.
   *
   * @param times The number of calling garbage collector
   */
  public static void callGCMultipleTimes(int times) {
    for (int i = 0; i < times; ++i) {
      JavaRuntime.gc();
    }
  }

  /**
   * Returns the maximum heap size in bytes.
   *
   * @return The maximum heap size in bytes
   */
  public static long maxMemoryBytes() {
    return JavaRuntime.maxMemory();
  }

  /**
   * Returns the available heap size in bytes.
   *
   * @return The available heap size in bytes.
   * @see <a href="https://stackoverflow.com/a/12807848/2034780">https://stackoverflow.com/a/12807848/2034780</a>
   */
  public static long availableMemoryBytes() {
    return JavaRuntime.maxMemory() - JavaRuntime.totalMemory() + JavaRuntime.freeMemory();
  }

  /**
   * Converts the given memory size from bytes to mebibytes.
   *
   * @param bytes A size of memory in bytes
   * @return The size of memory in mebibytes
   */
  public static long bytesToMebibytes(final long bytes) {
    return (long) (bytes / 1024.0 / 1024.0);
  }
}
