/*
 * PegasusN: Peta-Scale Graph Mining System (Pegasus v3.0)
 * Authors: Chiwan Park, Ha-Myung Park, U Kang
 *
 * Copyright (c) 2018, Ha-Myung Park, Chiwan Park, and U Kang
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Seoul National University nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * -------------------------------------------------------------------------
 * File: SeedGenerator.java
 * - Generate an input file for the subgraph enumeration step of PSE
 * Version: 3.0
 */


package pegasus.spark.subgraph;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

public class SeedGenerator {
	
	public static void main(String[] args) {
	}

	public static long[] subproblemGroupIds(int numColors, int numPatternNodes) {
		long[] interProblemIds = Combination.get(numColors,numPatternNodes);
		long[] intraProblemIds = Combination.get(numColors,numPatternNodes-1);

		long[] res = new long[interProblemIds.length + intraProblemIds.length];

		System.arraycopy(interProblemIds,0,res,0,interProblemIds.length);
		System.arraycopy(intraProblemIds,0,res,interProblemIds.length,intraProblemIds.length);

		return res;

	}

	public static long[] subproblemIds(int numColors, int numPatternNodes) {
		return Combination.get(numColors,numPatternNodes);
	}

	public static long[][] dual(int numColors, int numPatternNodes) {
		
		long[] interProblemIds = Combination.get(numColors,numPatternNodes);
		long[] intraProblemIds = Combination.get(numColors,numPatternNodes-1);

		@SuppressWarnings("unchecked")
		List<Long>[] intraProblems = (List<Long>[]) new ArrayList[intraProblemIds.length];
		for(int i = 0; i < intraProblemIds.length; i++)
			intraProblems[i] = new ArrayList<Long>();
		
		
		for(int c=numPatternNodes-1; c > 0; c--) {

			long[] problemIds = Combination.get(numColors, c);

			for (int i = 0; i < problemIds.length; i++) {

				long problemId = problemIds[i];

				int minNumProblems = Integer.MAX_VALUE;
				for (int j = 0; j < intraProblemIds.length; j++) {
					if ((intraProblemIds[j] & problemId) == problemId)
						minNumProblems = Math.min(minNumProblems, intraProblems[j].size());
				}

				List<Integer> minIndexes = new ArrayList<>();
				for (int j = 0; j < intraProblemIds.length; j++) {
					if ((intraProblemIds[j] & problemId) == problemId && intraProblems[j].size() == minNumProblems)
						minIndexes.add(j);
				}

				int minIndex = minIndexes.get(new Random().nextInt(minIndexes.size()));

				intraProblems[minIndex].add(problemId);
			}

		}
		
		List<long[]> res = new ArrayList<>();

		for(int i=0; i<interProblemIds.length; i++){
			res.add(new long[]{interProblemIds[i]});
		}

		for(int i=0; i<intraProblems.length; i++){
			res.add(intraProblems[i].stream().mapToLong(x -> x).toArray());
		}

		return res.toArray(new long[0][]);
	}
	
	
	private static class Combination{
		
		private static int n;
		private static int r;
		private static long[] items;
		private static int idx_item;
		
		public static long[] get(int n, int r){
			assert(n < 64);

			Combination.n = n;
			Combination.r = r;
			
			int numItems = 1;
			for(int i = 0; i < r; i++){
				numItems *= n-i;
				numItems /= i+1;
			}
			
			items = new long[numItems];
			idx_item = 0;

			dfs(0);
			
			return items;
		}
		
		private static void dfs(long item){
			if(Long.bitCount(item) == r){
				items[idx_item++] = item; //add item
			}
			else{
				//find the position of msb(most significant bit) + 1
				int i;
				for(i=0; i < n && (item >>> i ) > 0; i++);
				
				for(; i < n; i++){
					dfs(item | (1l << i));
				}
			}
			
		}
		
	}
	
}
