/**
  * PegasusN: Peta-Scale Graph Mining System (Pegasus v3.0)
  * Authors: Chiwan Park, Ha-Myung Park, U Kang
  *
  * Copyright (c) 2018, Ha-Myung Park, Chiwan Park, and U Kang
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
  *     * Redistributions of source code must retain the above copyright
  *       notice, this list of conditions and the following disclaimer.
  *     * Redistributions in binary form must reproduce the above copyright
  *       notice, this list of conditions and the following disclaimer in the
  *       documentation and/or other materials provided with the distribution.
  *     * Neither the name of the Seoul National University nor the
  *       names of its contributors may be used to endorse or promote products
  *       derived from this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
  * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
  * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
  * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
  * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  * -------------------------------------------------------------------------
  * File: GraphLoader.scala
  * - Loads graph files from HDFS
  * Version: 3.0
  */
package pegasus.spark.common

import org.apache.spark.SparkContext
import org.apache.spark.rdd.RDD

object GraphLoader {
  def edgeListFile(
    sc: SparkContext,
    path: String,
    fieldDelimiter: String = "\t"
  ): RDD[(Long, Long)] = {
    sc.textFile(path)
      .mapPartitions(iter => {
        iter.map(line => {
          val point = line.indexOf(fieldDelimiter)
          (line.substring(0, point).toLong, line.substring(point + 1).toLong)
        })
      }, preservesPartitioning = true)
  }

  def weightedEdgeListFile(
    sc: SparkContext,
    path: String,
    fieldDelimiter: String = "\t"
  ): RDD[(Long, Long, Double)] = {
    sc.textFile(path)
      .mapPartitions(iter => {
        iter.map(line => {
          val point1 = line.indexOf(fieldDelimiter)
          val point2 = line.indexOf(fieldDelimiter, point1 + 1)
          val src = line.substring(0, point1).toLong
          val dst = line.substring(point1 + 1, point2).toLong
          val weight = line.substring(point2 + 1).toDouble
          (src, dst, weight)
        })
      })
  }
}
