/**
  * PegasusN: Peta-Scale Graph Mining System (Pegasus v3.0)
  * Authors: Chiwan Park, Ha-Myung Park, U Kang
  *
  * Copyright (c) 2018, Ha-Myung Park, Chiwan Park, and U Kang
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
  *     * Redistributions of source code must retain the above copyright
  *       notice, this list of conditions and the following disclaimer.
  *     * Redistributions in binary form must reproduce the above copyright
  *       notice, this list of conditions and the following disclaimer in the
  *       documentation and/or other materials provided with the distribution.
  *     * Neither the name of the Seoul National University nor the
  *       names of its contributors may be used to endorse or promote products
  *       derived from this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
  * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
  * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
  * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
  * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  * -------------------------------------------------------------------------
  * File: BufferedReader.scala
  * - Reads file with buffer
  * Version: 3.0
  */
package pegasus.spark.common.io

import scala.reflect.ClassTag

class BufferedReader[@specialized(Int, Long, Double) T: ClassTag](reader: FileReader[T])
  extends Iterator[T] with AutoCloseable {
  private var _head: T = _
  private var headDefined = false

  /**
    * Returns the first element of file.
    *
    * @return The first element of file
    */
  def head: T = {
    if (!headDefined) {
      _head = reader.next()
      headDefined = true
    }

    _head
  }

  /**
    * Returns whether there is next element or not.
    *
    * @return true if there is next element, otherwise false
    */
  override def hasNext: Boolean = headDefined || reader.hasNext

  /**
    * Returns the next element.
    *
    * @return The next element
    */
  override def next(): T = {
    if (headDefined) {
      headDefined = false
      _head
    } else reader.next()
  }

  /**
    * Closes this file.
    */
  override def close(): Unit = reader.close()
}
