package pegasus.spark.common.io

import java.io.File
import java.nio.file.{Files, Paths}
import java.util
import java.util.Random

import scala.collection.mutable.ArrayBuffer

/**
  * Created by hmpark on 17. 6. 8.
  * !IMPORTANT! ExternalSorter does not support simultaneous (multicore) running.
  */
object ExternalSorter {
  var bufferSize = 0
  var values: Array[Long] = _
  var array_size: Int = 0

  def init(basePaths: Array[String]){
    bufferSize = ((availableMemoryBytes(true) / java.lang.Long.BYTES) / 8).toInt
    values = new Array[Long](bufferSize)
    LocalPaths.setBasePaths(basePaths)
  }

  def sort(it: Iterator[Long], basePaths: Array[String]): Iterator[Long] = {

    if(bufferSize == 0) init(basePaths)

    val inPaths = ExternalSorter.localSort(it)

    val first = values.iterator.take(array_size).foldRight(List.empty[Long]){
      case (a, b) =>
        if (b.nonEmpty && b.head == a)
          b
        else
          a :: b
    }.iterator

    if(inPaths.length == 0){
      first
    }
    else{
      val mergefirst = LocalPaths.getTempPath("mergefirst")
      mergeTwo(first,
        new LongIteratorFromDirectInput(inPaths(inPaths.length-1)),
        mergefirst)

      inPaths(inPaths.length-1) = mergefirst

      new LongIteratorFromDirectInput(ExternalSorter.mergeAll(inPaths))
    }

  }

  def localSort(it: Iterator[Long]): Array[String] = {

    array_size = 0

    val inPaths = new ArrayBuffer[String]

    var remain = true

    while(it.hasNext){

      val it_this = it.take(bufferSize)

      var i = 0
      it_this.foreach{ x =>
        values(i) = x
        i += 1
      }


      array_size = i

      util.Arrays.sort(values, 0, array_size)

      if(it.hasNext){
        val tPath = LocalPaths.getTempPath("localsort")

        val out = new DirectWriter(tPath)

        var prev = -1L
        (0 until array_size).foreach{ idx =>
          val now = values(idx)
          if(prev != now){
            out.write(now)
          }
          prev = now
        }

        out.close()

        inPaths += tPath
      }
      else{

      }
    }

    inPaths.toArray

  }

  private def mergeTwo(in1: Iterator[Long], in2: Iterator[Long], outPath: String) = {

    val out = new DirectWriter(outPath)
    var prev = -1L
    var u = 0L
    var v = 0L
    u = in1.next
    v = in2.next

    var remain = true

    def write(x: Long): Unit ={
      if(prev != x){
        out.write(x)
        prev = x
      }
    }

    while(remain){
      if(u < v){
        write(u)
        if(in1.hasNext) u = in1.next
        else{
          write(v)
          remain = false
        }
      }
      else if (u > v){
        write(v)
        if(in2.hasNext) v = in2.next
        else{
          write(u)
          remain = false
        }
      }
    }

    while(in1.hasNext){
      write(in1.next())
    }

    while(in2.hasNext){
      write(in2.next())
    }

    out.close()

  }

  private def mergeAll(inPaths: Array[String]) : String = {
    val queue = scala.collection.mutable.Queue[String]()
    queue ++= inPaths

    while (queue.size > 1) {
      val in1Path = queue.dequeue
      val in2Path = queue.dequeue
      val outPath = LocalPaths.getTempPath("mergeall")
      mergeTwo(new LongIteratorFromDirectInput(in1Path), new LongIteratorFromDirectInput(in2Path), outPath)
      Files.delete(Paths.get(in1Path))
      Files.delete(Paths.get(in2Path))
      queue.enqueue(outPath)
    }

    queue.dequeue
  }

  private def availableMemoryBytes(gc: Boolean) = {
    if (gc) {
      System.gc(); System.gc()
    }
    Runtime.getRuntime.maxMemory - (Runtime.getRuntime.totalMemory - Runtime.getRuntime.freeMemory)
  }

  private[io] object LocalPaths {
    private val rand: Random = new Random
    private var basePaths: Array[String] = _

    private[io] def setBasePaths(basePaths: Array[String]): Unit = {
      for (tmpDir <- basePaths) {
        new File(tmpDir).mkdirs
      }
      LocalPaths.basePaths = basePaths
    }

    private def getNextBasePath: String = basePaths(rand.nextInt(basePaths.length))

    private[io] def getTempPath(prefix: String): String = {
      var res: String = null
      do {
        val t: Int = rand.nextInt(Integer.MAX_VALUE)
        res = getNextBasePath + "/" + prefix + "-" + t
      } while (Files.exists(Paths.get(res)))
      res
    }
  }
}
