/**
  * PegasusN: Peta-Scale Graph Mining System (Pegasus v3.0)
  * Authors: Chiwan Park, Ha-Myung Park, U Kang
  *
  * Copyright (c) 2018, Ha-Myung Park, Chiwan Park, and U Kang
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
  *     * Redistributions of source code must retain the above copyright
  *       notice, this list of conditions and the following disclaimer.
  *     * Redistributions in binary form must reproduce the above copyright
  *       notice, this list of conditions and the following disclaimer in the
  *       documentation and/or other materials provided with the distribution.
  *     * Neither the name of the Seoul National University nor the
  *       names of its contributors may be used to endorse or promote products
  *       derived from this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
  * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
  * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
  * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
  * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  * -------------------------------------------------------------------------
  * File: MemoryGroupedIterator.scala
  * - An iterator object that groups records in memory.
  * Version: 3.0
  */
package pegasus.spark.common.io

import pegasus.spark.common.Constants

import scala.reflect.ClassTag

class MemoryGroupedIterator[@specialized(Int, Long, Double) T: ClassTag](
  keys: Array[Int],
  values: Array[T],
  numInstances: Int
) extends Iterator[(Int, Int, Array[T])] {
  private var current = 0
  private var tempArr: Array[T] = Array.ofDim[T](Constants.InitArrayWritableSize)

  override def hasNext: Boolean = current < numInstances

  override def next(): (Int, Int, Array[T]) = {
    val key = keys(current)

    var i = current
    var flag = i < numInstances
    var count = 0
    while (flag) {
      if (key == keys(i)) {
        if (count >= tempArr.length) {
          val newValue = Array.ofDim[T](tempArr.length * 2)
          Array.copy(tempArr, 0, newValue, 0, tempArr.length)
          tempArr = newValue
        }
        tempArr(count) = values(i)
        count += 1
      }
      i += 1
      flag = i < numInstances && key == keys(i)
    }
    current = i

    (key, count, tempArr.slice(0, count))
  }
}
