/**
  * PegasusN: Peta-Scale Graph Mining System (Pegasus v3.0)
  * Authors: Chiwan Park, Ha-Myung Park, U Kang
  *
  * Copyright (c) 2018, Ha-Myung Park, Chiwan Park, and U Kang
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
  *     * Redistributions of source code must retain the above copyright
  *       notice, this list of conditions and the following disclaimer.
  *     * Redistributions in binary form must reproduce the above copyright
  *       notice, this list of conditions and the following disclaimer in the
  *       documentation and/or other materials provided with the distribution.
  *     * Neither the name of the Seoul National University nor the
  *       names of its contributors may be used to endorse or promote products
  *       derived from this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
  * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
  * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
  * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
  * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  * -------------------------------------------------------------------------
  * File: WeightedMatrixReader.scala
  * - Reads a weighted matrix from file
  * Version: 3.0
  */
package pegasus.spark.common.io

import pegasus.spark.common.Constants

class WeightedMatrixReader(path: String) extends MatrixReader[(Int, Double)](path) {
  private var rows = Array.ofDim[(Int, Double)](Constants.InitArrayWritableSize)

  override protected def readRows(): Array[(Int, Double)] = {
    val headInt = dataIn.readInt()
    val headDouble = dataIn.readDouble()

    if ((headInt & Int.MinValue) != 0) {
      val length = dataIn.readInt()
      val rowsLocal = if (rows.length < length) {
        rows = Array.ofDim[(Int, Double)](length)
        rows
      } else {
        rows
      }

      var i = 1
      rowsLocal(0) = (headInt & Int.MaxValue, headDouble)
      while (i < length) {
        rowsLocal(i) = (dataIn.readInt(), dataIn.readDouble())
        i += 1
      }
      rowsLocal.slice(0, length)
    } else {
      Array((headInt & Int.MaxValue, headDouble))
    }
  }
}
