/**
  * PegasusN: Peta-Scale Graph Mining System (Pegasus v3.0)
  * Authors: Chiwan Park, Ha-Myung Park, U Kang
  *
  * Copyright (c) 2018, Ha-Myung Park, Chiwan Park, and U Kang
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
  *     * Redistributions of source code must retain the above copyright
  *       notice, this list of conditions and the following disclaimer.
  *     * Redistributions in binary form must reproduce the above copyright
  *       notice, this list of conditions and the following disclaimer in the
  *       documentation and/or other materials provided with the distribution.
  *     * Neither the name of the Seoul National University nor the
  *       names of its contributors may be used to endorse or promote products
  *       derived from this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
  * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
  * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
  * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
  * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  * -------------------------------------------------------------------------
  * File: BitSet.scala
  * - A compressed bit set
  * Version: 3.0
  */
package pegasus.spark.common.types

class BitSet(val cardinality: Int) {
  
  import BitSet._

  private val bits = Array.ofDim[Long](math.ceil(cardinality / 64.0).toInt)

  def arraySize: Int = bits.length

  def contains(id: Int): Boolean = {
    val pos = id / 64
    val mask = Masks(id % 64)
    (bits(pos) & mask) != 0
  }

  def add(id: Int): Unit = {
    val pos = id / 64
    val mask = Masks(id % 64)
    bits(pos) |= mask
  }

  def remove(id: Int): Unit = {
    val pos = id / 64
    val mask = Masks(id % 64)
    bits(pos) &= ~mask
  }

  def clear(): Unit = {
    java.util.Arrays.fill(bits, 0L)
  }
}

object BitSet {
  val Masks: Array[Long] = Array.tabulate[Long](64)(i => 1L << i)
}
