/**
  * PegasusN: Peta-Scale Graph Mining System (Pegasus v3.0)
  * Authors: Chiwan Park, Ha-Myung Park, U Kang
  *
  * Copyright (c) 2018, Ha-Myung Park, Chiwan Park, and U Kang
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
  *     * Redistributions of source code must retain the above copyright
  *       notice, this list of conditions and the following disclaimer.
  *     * Redistributions in binary form must reproduce the above copyright
  *       notice, this list of conditions and the following disclaimer in the
  *       documentation and/or other materials provided with the distribution.
  *     * Neither the name of the Seoul National University nor the
  *       names of its contributors may be used to endorse or promote products
  *       derived from this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
  * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
  * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
  * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
  * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  * -------------------------------------------------------------------------
  * File: VectorBlock.scala
  * - A base implementation of vector block
  * Version: 3.0
  */
package pegasus.spark.common.types

abstract class VectorBlock[@specialized(Int, Long, Double) T](
  numBlocks: Int,
  numVertices: Long,
  private var _blockId: Int
) {
  val size: Int = math.ceil(numVertices / numBlocks.toDouble).toInt
  private val containsSet = new BitSet(size)

  final def contains(i: Int): Boolean = containsSet.contains(i)

  protected final def mark(i: Int): Unit = containsSet.add(i)

  protected final def clearContainsSet(): Unit = containsSet.clear()

  def blockId: Int = _blockId

  def setBlockId(blockId: Int): Unit = {
    _blockId = blockId
  }

  def get(i: Int): T

  def put(i: Int, v: T): Unit

  def addTo(i: Int, v: T): Unit

  def clear(): Unit

  def iterator(onlyNonzero: Boolean): VectorIterator[T]

  def iterator(onlyNonZero: Boolean, filterSet: BitSet): VectorIterator[T]
}
