/**
  * PegasusN: Peta-Scale Graph Mining System (Pegasus v3.0)
  * Authors: Chiwan Park, Ha-Myung Park, U Kang
  *
  * Copyright (c) 2018, Ha-Myung Park, Chiwan Park, and U Kang
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
  *     * Redistributions of source code must retain the above copyright
  *       notice, this list of conditions and the following disclaimer.
  *     * Redistributions in binary form must reproduce the above copyright
  *       notice, this list of conditions and the following disclaimer in the
  *       documentation and/or other materials provided with the distribution.
  *     * Neither the name of the Seoul National University nor the
  *       names of its contributors may be used to endorse or promote products
  *       derived from this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
  * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
  * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
  * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
  * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  * -------------------------------------------------------------------------
  * File: WeightedMatrixRows.scala
  * - A row that contains weighted matrix elements
  * Version: 3.0
  */
package pegasus.spark.common.types

import pegasus.spark.common.Constants

class WeightedMatrixRows extends MatrixRows[(Int, Double)] {
  private var rows = Array.ofDim[Int](Constants.InitArrayWritableSize)
  private var values = Array.ofDim[Double](Constants.InitArrayWritableSize)
  private var length = 0

  override final def size(): Int = length

  override final def row(i: Int): (Int, Double) = (rows(i), values(i))

  def setRows(rows: Array[Int], values: Array[Double], begin: Int, end: Int): Unit = {
    val length = end - begin
    if (this.rows.length < length) {
      this.rows = Array.ofDim[Int](length)
      this.values = Array.ofDim[Double](length)
    }

    var i = begin
    while (i < end) {
      this.rows(i - begin) = rows(i)
      this.values(i - begin) = values(i)
      i += 1
    }

    this.length = length
  }

  override final def setRows(rows: Array[(Int, Double)], begin: Int, end: Int): Unit = {
    setRows(rows.map(_._1), rows.map(_._2), begin, end)
  }
}
