package pegasus.spark.cc.deprecated.beta

import org.apache.log4j.{Level, Logger}
import org.apache.spark.{SparkConf, SparkContext}
import org.scalatest.{FlatSpec, Matchers}
import pegasus.spark.cc.PACCBasic

/**
  * Created by hmpark on 17. 3. 17.
  */
class PACCBasicSpec extends FlatSpec with Matchers {

  val logger = Logger.getLogger(getClass)
  logger.setLevel(Level.INFO)

  "largeStar" should "work correctly on an example data" in {

    val conf = new SparkConf().setAppName("largeStarTest").setMaster("local[2]")

    val sc = new SparkContext(conf)

    val inputdata = Seq[(Long, Long)]((~2, 1), (~3, 1), (~4, 1), (6, 5), (7,6), (8,6), (9,6), (9,7))
    val inputRDD = sc.parallelize(inputdata)

    val paccbasic = new PACCBasic()

    val (lout, lother, numChanges, numOut, numCC, numIn) = paccbasic.largeStar(inputRDD, 2)

    Set((6,5), (7,5), (9,5), (9,7), (8,6)).equals(lout.collect().toSet) should be (true)

    Set((2,1), (3,1), (4,1)).equals(lother.collect().toSet) should be (true)

    numChanges should be (2)
    numOut should be (5)
    numIn should be (0)
    numCC should be (3)

    println(f"num changes: $numChanges, numOut: $numOut, numCC: $numCC, numIn: $numIn")

    sc.stop()

  }

  "smallStar" should "work correctly on an example data" in {

    val conf = new SparkConf().setAppName("largeStarTest").setMaster("local[2]")

    val sc = new SparkContext(conf)

    val inputdata = Seq[(Long, Long)]((1,6), (2,6), (4,6), (2,5), (2,3), (5,7))
    val inputRDD = sc.parallelize(inputdata)

    val paccbasic = new PACCBasic()

    val (sout, sin, numChanges, numOut, numIn) = paccbasic.smallStar(inputRDD, 2)

    Set((2,1),(4,2),(~3,2),(5,2)).equals(sout.collect().toSet) should be (true)

    Set((6,2),(7,5)).equals(sin.collect().toSet) should be (true)

    numChanges should be (2)
    numOut should be (4)
    numIn should be (2)

    println(f"num changes: $numChanges, numOut: $numOut, numIn: $numIn")

    sc.stop()

  }

  "PACCBasic" should "output 4157 nodes from GCC in grqc" in {
    val path = getClass.getResource("/graphs/grqc").toString
    val lt = 0
    val np = 10

    logger.info(f"test( data: $path, localThreahold: $lt, numPartition: $np )")

    val conf = new SparkConf().setAppName("PACC_D-test").setMaster("local[1]")
      .set("spark.serializer", "org.apache.spark.serializer.KryoSerializer")
      .set("spark.kryo.registrator", "pegasus.spark.subgraph.MyRegistrator")
      .set("spark.kryoserializer.buffer", "24m")

    val paccbasic = new PACCBasic()

    val sc = new SparkContext(conf)
    val numNodesInGCC = paccbasic.run(path, lt, np, sc)
      .map{case (u, c) => (c, u)}
      .groupByKey()
      .map{case (c, nodes) => nodes.size}
      .collect().max

    numNodesInGCC should be (4157)
  }

}
