package pegasus.spark.cc.deprecated.beta

import org.apache.log4j.{Level, Logger}
import org.apache.spark.sql.SparkSession
import org.apache.spark.{SparkConf, SparkContext}
import org.scalatest.{FlatSpec, Matchers}

/**
  * Created by hmpark on 17. 3. 17.
  */
class PACCTestSpec extends FlatSpec with Matchers {

  val logger = Logger.getLogger(getClass)
  logger.setLevel(Level.INFO)

  "largeStar" should "work correctly on an example data" in {

    val conf = new SparkConf().setAppName("largeStarTest").setMaster("local[1]")

    val ss = SparkSession.builder().appName("test").master("local[1]").getOrCreate()

    import ss.implicits._

    val inputdata = Seq[(Long, Long)]((~2, 1), (~3, 1), (~4, 1), (6, 5), (7,6), (8,6), (9,6), (9,7))

    val inDS = ss.createDataset(inputdata)

    val (lout, lother, numChanges, numOut, numCC, numIn) = PACCTest.largeStar(inDS, 2)

    lout.collect().toSet should be (Set((6,5), (7,5), (9,5), (9,7), (8,6)))

    Set((2,1), (3,1), (4,1)).equals(lother.collect().toSet) should be (true)

    numChanges should be (2)
    numOut should be (5)
    numIn should be (0)
    numCC should be (3)

    println(f"num changes: $numChanges, numOut: $numOut, numCC: $numCC, numIn: $numIn")

    ss.stop()

  }

  "smallStar" should "work correctly on an example data" in {


    val ss = SparkSession.builder().appName("test").master("local[2]").getOrCreate()

    import ss.implicits._

    val inputdata = Seq[(Long, Long)]((1,6), (2,6), (4,6), (2,5), (2,3), (5,7))
    val inDS = ss.createDataset(inputdata)

    val (sout, sin, numChanges, numOut, numIn) = PACCTest.smallStar(inDS, 2)

    Set((2,1),(4,2),(~3,2),(5,2)).equals(sout.collect().toSet) should be (true)

    Set((6,2),(7,5)).equals(sin.collect().toSet) should be (true)

    numChanges should be (2)
    numOut should be (4)
    numIn should be (2)

    println(f"num changes: $numChanges, numOut: $numOut, numIn: $numIn")

    ss.stop()

  }

  "PACCTest" should "output 4157 nodes from GCC in grqc" in {
    val path = getClass.getResource("/graphs/grqc").toString
    val lt = 0
    val np = 10

    logger.info(f"test( data: $path, localThreahold: $lt, numPartition: $np )")

//    val conf = new SparkConf().setAppName("PACC_D-test").setMaster("local[1]")
//      .set("spark.serializer", "org.apache.spark.serializer.KryoSerializer")
//      .set("spark.kryo.registrator", "pegasus.spark.subgraph.MyRegistrator")
//      .set("spark.kryoserializer.buffer", "24m")

    val ss = SparkSession.builder().appName("ttttest").master("local[1]").getOrCreate()

    import ss.implicits._
    import org.apache.spark.sql.functions._

    val numNodesInGCC = PACCTest.run(path, lt, np, ss)
      .groupBy($"_2").agg(count("*").alias("cnt")).select("cnt").as[Long].rdd.max








//      .groupByKey()
//      .map{case (c, nodes) => nodes.size}
//      .collect().max

    numNodesInGCC should be (4157)
  }

}
