package pegasus.spark.graphgen

import org.apache.hadoop.conf.Configuration
import org.apache.hadoop.fs.{FileSystem, Path}
import org.apache.spark.{SparkConf, SparkContext}
import org.scalatest.{FlatSpec, Matchers}


/**
  * Created by hmpark on 17. 3. 10.
  */
class KroneckerGeneratorSpec extends FlatSpec with Matchers {

  "KroneckerGenerator" should "run without error" in {

    val level = 3
    val numTasks = 5

    val seedGraph = Array((0,1), (0,2), (1,0), (1,1), (1,2), (2,0), (2,1))

    val conf = new SparkConf().setAppName(f"[Kronecker] lv: $level, seed: $seedGraph").setMaster("local[2]")
    val sc = new SparkContext(conf)

    val res = KroneckerGenerator.run(level, seedGraph, numTasks, sc).map{case (u, v) => u + "\t" + v}.foreach(println)

    sc.stop()

  }

  "KroneckerGenerator" should "run on a seed graph without error" in {

    val level = 3
    val numTasks = 5
    val outputPath = "kronecker_output"

    val inputParameter = Array(level.toString, numTasks.toString,
      getClass.getResource("/graphs/kronecker_seed").getPath, outputPath, "true")

    FileSystem.get(new Configuration()).delete(new Path(outputPath), true)

    KroneckerGenerator.main(inputParameter)

  }

}
