package pegasus.spark.graphgen

import java.io.File
import java.nio.file.{Files, Paths}

import org.apache.commons.io.FileUtils
import org.apache.spark.{SparkConf, SparkContext}
import org.sameersingh.scalaplot.Implicits.{output, xyChart}
import org.sameersingh.scalaplot.XYPlotStyle
import org.scalatest.{FlatSpec, Matchers}
import org.sameersingh.scalaplot.Implicits._
import org.sameersingh.scalaplot.Style.PointType



/**
  * Created by hmpark on 17. 3. 10.
  */
class RMATGeneratorSpec extends FlatSpec with Matchers {

  "RMATGenerator" should "run without error" in {

    val level = 18
    val numEdges = 2000000L
    val numTasks = 10
    val abcd = (0.5, 0.25, 0.15, 0.1)
    val noise = 0

//    val output = "test_rmat_graph"
//    FileUtils.forceDelete(new File(output))

    val conf = new SparkConf().setAppName(f"[RMAT] lv: $level, E: $numEdges, abcd: $abcd").setMaster("local[2]")
    val sc = new SparkContext(conf)

    val res = RMATGenerator.run(level, numEdges, numTasks, abcd, noise, sc).collect()
//    res.map{x => x._1 + "\t" + x._2}.saveAsTextFile("rmat_res")

    spyplot(res.toStream, "dddddddd")

    degcntplot(res.toStream, "dddddd22")




//    res.map(x => x._1 + "\t" + x._2).saveAsTextFile(output)

    sc.stop()

//    val writer = new PrintWriter(new File("Write.txt"))
//
//    val noise = 0.1
//
//    val noiseList = (0L until level).map(_ => Math.random() * 2 * noise - noise).toArray
//    val x = new RMATGenerator.RMAT(abcd, noiseList)
//
//    val y = x.generate(RMATGenerator.Region(0, 0, (1L << level) - 1, (1L << level) - 1, numEdges, level-1))
//
//
//
////    val x = RMATGenerator.doRMAT(
////      RMATGenerator.Region(0, 0, (1L << level) - 1, (1L << level) - 1, numEdges, level),
////      abcd)
//    y.foreach(t => writer.print(t._1 + "\t" + t._2 + "\n"))
//
//    writer.close()



//    sc.stop()

  }

  def spyplot(res:Stream[(Long, Long)], fileName:String): Unit = {

    val spy = res.toArray.toSeq
    val data = spy.map(_._1.toDouble) -> Seq( Y(spy.map(_._2.toDouble), style=XYPlotStyle.Dots) )

    output(PNG("res_png/spy/", fileName), xyChart(data) )

  }

  def degcntplot(res:Stream[(Long, Long)], fileName:String): Unit = {
    val degcnt = res.flatMap{x => Stream(x._1, x._2)}
      .groupBy(identity).mapValues(x=> x.length).values
      .groupBy(identity).mapValues(x => x.size).toSeq.sortBy(_._1)

    val data = degcnt.map{_._1.toDouble} ->
      Seq(Y(degcnt.map{_._2.toDouble}, style=XYPlotStyle.Points, pt=Option(PointType fullO), ps=Option(1)))

    output(PNG("res_png/degcnt/", fileName),
      xyChart(data, x= Axis(log = true), y = Axis(log = true, range = Option((0.9, degcnt.map{_._2}.max)) ))
    )
  }

}
