package pegasus.spark.graphgen

import java.io.{BufferedWriter, FileWriter}

import org.sameersingh.scalaplot.Implicits._
import org.apache.spark.{SparkConf, SparkContext}
import org.sameersingh.scalaplot.Style.PointType
import org.sameersingh.scalaplot.{MemXYSeries, XYData, XYPlotStyle}
import org.scalatest.{FlatSpec, Matchers}


/**
  * Created by hmpark on 17. 3. 10.
  */
class RMATSingleSpec extends FlatSpec with Matchers {

  val level = 18
  val numEdges = 2000000L
  val abcd = (0.5, 0.25, 0.15, 0.1)
  val noise = 0

  def degcntplot(res:Stream[(Long, Long)], fileName:String): Unit = {
    val degcnt = res.flatMap{x => Stream(x._1, x._2)}
      .groupBy(identity).mapValues(x=> x.length).values
      .groupBy(identity).mapValues(x => x.size).toSeq.sortBy(_._1)

    val data = degcnt.map{_._1.toDouble} ->
      Seq(Y(degcnt.map{_._2.toDouble}, style=XYPlotStyle.Points, pt=Option(PointType fullO), ps=Option(1)))

    output(PNG("res_png/degcnt/", fileName),
      xyChart(data, x= Axis(log = true), y = Axis(log = true, range = Option((0.9, degcnt.map{_._2}.max)) ))
    )
  }

  def spyplot(res:Stream[(Long, Long)], fileName:String): Unit = {

    val spy = res.toArray.toSeq
    val data = spy.map(_._1.toDouble) -> Seq( Y(spy.map(_._2.toDouble), style=XYPlotStyle.Dots) )

    output(PNG("res_png/spy/", fileName), xyChart(data) )

  }

  def write(res:Stream[(Long, Long)], fileName:String): Unit = {
    val bw = new BufferedWriter(new FileWriter(fileName))
    res.foreach{case (x, y) => bw.write(x + "\t" + y + "\n")}
    bw.close()
  }

  "RMATOriginal" should "run without error" in {
    val res = RMATSingle.generateAsOriginal(level,numEdges,0,abcd)
    degcntplot(res, "rmat_origin_res.degcnt")
    spyplot(res, "rmat_origin_res.spy")
  }

  "TeG" should "run without error" in {
    val res = RMATSingle.generateByTeg(level,numEdges,0, 100000,abcd)
    degcntplot(res, "rmat_teg_res.degcnt")
    spyplot(res, "rmat_teg_res.spy")
  }

  "Multinomial" should "run without error" in {
    val res = RMATSingle.generateWithMultinomial(level,numEdges,0,100000,abcd)
    degcntplot(res, "rmat_mult_res.degcnt")
    spyplot(res, "rmat_mult_res.spy")
  }

  "MultinomialSrand" should "run without error" in {
    val res = RMATSingle.genMultSRand(level,numEdges,0,100000,abcd)
    degcntplot(res, "rmat_mults_res.degcnt")
        spyplot(res, "rmat_mult_res.spy")
  }

//
//  "LimitedLevelMultinomial" should "run without error" in {
//    val res = RMATSingle.generateWithLimitLevelMultinomial(level,numEdges,0,level-6,abcd)
//    degcntplot(res, "rmat_limit_res.degcnt")
//  }
//
//  "LimitedTeG" should "run without error" in {
//    val res = RMATSingle.generateWithLimitLevelTeG(level,numEdges,0,level-6,abcd)
//    degcntplot(res, "rmat_tegl_res.degcnt")
//  }
////
//  "LimitedLevelMultinomialWithNoise" should "run without error" in {
//    val res = RMATSingle.generateWithLimitLevelMultinomial(level,numEdges,0.15,level-6,abcd)
//    degcntplot(res, "rmat_limit_noise_0.1_res.degcnt")
//  }

}
