package pegasus.spark.triangle

import java.io.File
import java.util.StringTokenizer

import org.apache.commons.io.FileUtils
import org.apache.hadoop.conf.Configuration
import org.apache.hadoop.fs.{FileSystem, Path}
import org.apache.log4j.{Level, Logger}
import org.apache.spark.{SparkConf, SparkContext}
import org.scalatest.{FlatSpec, Matchers}
import pegasus.spark.subgraph.EdgeOrder

import scala.io.Source

/**
  * Created by hmpark on 17. 3. 10.
  */
class PTESpec extends FlatSpec with Matchers {

  Logger.getLogger("org").setLevel(Level.OFF)
  Logger.getLogger("akka").setLevel(Level.OFF)

  "PTE" should "output right triangle numbers" in {

    println(getClass.getResource("/graphs/facebook"))

    val inputPath = getClass.getResource("/graphs/facebook").toString
    val basePath = inputPath + ".test"
    val oedgePath = basePath + "/facebook.oedge"
    val numColors: Short = 5

    EdgeOrder.run(inputPath, oedgePath, local = true)

    val pte = new PTE()

    val conf = new SparkConf().setAppName(pte.getClass.getSimpleName + "-test").setMaster("local[1]")
      .set("spark.serializer", "org.apache.spark.serializer.KryoSerializer")
      .set("spark.kryo.registrator", "pegasus.spark.subgraph.MyRegistrator")
      .set("spark.kryoserializer.buffer", "24m")

    val sc = new SparkContext(conf)

    val triangles = pte.run(oedgePath, numColors, sc)

    triangles.count() should be (1612010L)

    FileUtils.forceDeleteOnExit(new File(basePath))
    FileSystem.get(new Configuration()).deleteOnExit(new Path(basePath))

  }

//  "intersectIterator" should "work correctly" in {
//    val pte = new PTE()
//    val uAdj = Array(1,2,3,4,5,6,7,8,9)
//    val vAdj = Array(3,5,7,8,9,10,11)
//    val it = pte.intersectIterator(uAdj, vAdj)
//  }




}
